from telebot import types
from minaibot.answers import cache


def start(bot, message, server):
    markup = types.ReplyKeyboardMarkup(resize_keyboard=True)
    btn1 = types.KeyboardButton("/start")
    btn2 = types.KeyboardButton("/stop")
    markup.add(btn1, btn2)
    if message.chat.id in cache and cache[message.chat.id] == message.text:
        bot.send_message(message.chat.id,
                         text=f"Мы уже здаровались..",
                         reply_markup=markup)
    else:
        bot.send_message(message.chat.id,
                         text=f"Привет, я бот, созданный через фреймворк minaibot!",
                         reply_markup=markup)
    cache[message.chat.id] = message.text


def stop(bot, message, server):
    markup = types.ReplyKeyboardMarkup(resize_keyboard=True)
    btn1 = types.KeyboardButton("/start")
    btn2 = types.KeyboardButton("/stop")
    markup.add(btn1, btn2)
    bot.send_message(message.chat.id,
                     text=f"Меня не остановить!",
                     reply_markup=markup)



from minaibot.core.router import path
from minaibot.answers.start import start, stop

routes = [
    path(pathname="start", msgtype="text", text="/start", textcheckmethod="string", answer=start),
    path(pathname="stop", msgtype="text", text="/stop", textcheckmethod="string", answer=stop),
]



from minaibot.bot.config import PERMISSION_DENIED_MESSAGE


def check_permissions(bot, message, server):
    def decorator(view_func):
        def wrapped(*args, **kwargs):
            if not has_permissions(message, server):
                bot.send_message(message.chat.id,
                                 text=PERMISSION_DENIED_MESSAGE % (message.from_user.first_name,
                                                                       "@nikita_fomichev"))
                return
            return view_func(*args, **kwargs)
        return wrapped
    return decorator


def has_permissions(message, server):
    return server.check_permission(message)




import re
from telebot.types import Optional
from minaibot.bot import server


class Router:
    def __init__(self, bot):
        self.bot = bot
        self.message = None
        self.pathbefore = {}
        self.server = server()

    def __check_path(self, _path):
        if self.message.content_type == _path[1]:
            if _path[1] == 'text':
                if _path[4] == "string":
                    if _path[5]:
                        if _path[5] == self.pathbefore[self.message.chat.id]:
                            return self.message.text == _path[3]
                    else:
                        return self.message.text == _path[3]
                elif _path[4] == 'regex':
                    if _path[5]:
                        if _path[5] == self.pathbefore[self.message.chat.id]:
                            return re.match(_path[3], self.message.text)
                        else:
                            return False
                    else:
                        return re.match(_path[3], self.message.text)
                else:
                    return False
            return False
        return False

    def routing(self, message, routes: list[callable]):
        self.message = message
        if routes:
            for _path in routes:
                if self.__check_path(_path):
                    _path[2](self.bot, self.message, self.server)
                    self.pathbefore[self.message.chat.id] = _path[0]
                    break


def path(
        pathname: str,
        msgtype: str,
        answer: callable,
        text: Optional[str] = None,
        textcheckmethod: Optional[str] = None,
        pathbefore: Optional[str] = None
):
    """
    Функция для обозначения пути во время диалога с ботом
    :param pathname: Уникальное название пути
    :param msgtype: Тип сообщения:
    :param answer: Функция ответа
    :param text: Текст сообщения
    :param textcheckmethod: Метод проверки текста сообщения: "string" или "regex"
    :param pathbefore: Название пути, который было до текущего
    :return:
    """
    return pathname, msgtype, answer, text, textcheckmethod, pathbefore



import requests
import types


class Server:
    url = "http://httpbin.org/get"
    headers = dict()

    def request(self, method, payload):
        return requests.request(method, self.url, data=payload)

    def check_permission(self, *args, **kwargs):
        """
        You can override this method in subclasses
        :return: True / False
        """
        return True

    def call_subclass_method(self):
        raise NotImplementedError("Subclasses should implement this method!")


import telebot
from minaibot.bot.config import TOKEN, DEBUG_MESSAGE
from .decorators import check_permissions
from .router import Router
from minaibot.bot.routes import routes


bot = telebot.TeleBot(TOKEN)
router = Router(bot)


@bot.message_handler(content_types=['photo', 'document', 'video', 'text'])
def handle_message(message):
    if DEBUG_MESSAGE:
        print(message.__dict__)

    @check_permissions(bot, message, router.server)
    def get_routes():
        return routes
    router.routing(message, get_routes())


bot.polling(none_stop=True)



import os
import shutil


def create_directories_and_files():
    source_bot_dir = os.path.join(os.path.dirname(__file__), 'minaibot', 'bot')
    target_bot_dir = os.path.join(os.getcwd(), 'bot')

    if not os.path.exists(target_bot_dir):
        os.makedirs(target_bot_dir)

    for item in os.listdir(source_bot_dir):
        s = os.path.join(source_bot_dir, item)
        d = os.path.join(target_bot_dir, item)
        if os.path.isdir(s):
            shutil.copytree(s, d, dirs_exist_ok=True)
        else:
            shutil.copy2(s, d)


if __name__ == "__main__":
    create_directories_and_files()



from setuptools import setup, find_packages
import os

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name="minaibot",
    version="0.1.2",
    description="BotFramework",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    author="Nikita Minai",
    author_email="nikita.minai@ya.ru",
    url="https://github.com/nikita-minai/minaibot",
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        "requests",
        "pyTelegramBotAPI",
        "telebot",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    entry_points={
        'console_scripts': [
            'minaibot_setup=post_install:create_directories_and_files',
        ],
    },
)
